"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const metadata_1 = require("../metadata");
const utils_1 = require("../utils");
const entity_1 = require("../entity");
function OneToMany(entity, mappedBy, options = {}) {
    return createOneToDecorator(entity, mappedBy, options, entity_1.ReferenceType.ONE_TO_MANY);
}
exports.OneToMany = OneToMany;
function createOneToDecorator(entity, mappedBy, options, reference) {
    return function (target, propertyName) {
        options = utils_1.Utils.isObject(entity) ? entity : Object.assign(Object.assign({}, options), { entity, mappedBy });
        const meta = metadata_1.MetadataStorage.getMetadata(target.constructor.name);
        utils_1.Utils.lookupPathFromDecorator(meta);
        if (reference === entity_1.ReferenceType.ONE_TO_MANY) {
            if (options.fk) {
                throw new Error(`@OneToMany({ fk })' is deprecated, use 'mappedBy' instead in '${target.constructor.name}.${propertyName}'`);
            }
        }
        const prop = {
            name: propertyName,
            reference,
            cascade: [entity_1.Cascade.PERSIST, entity_1.Cascade.MERGE],
        };
        Object.assign(prop, options);
        if (reference === entity_1.ReferenceType.ONE_TO_ONE) {
            utils_1.Utils.defaultValue(prop, 'nullable', prop.cascade.includes(entity_1.Cascade.REMOVE) || prop.cascade.includes(entity_1.Cascade.ALL));
            prop.owner = prop.owner || !!prop.inversedBy || !prop.mappedBy;
            prop.unique = prop.owner;
            if (prop.owner && options.mappedBy) {
                utils_1.Utils.renameKey(prop, 'mappedBy', 'inversedBy');
            }
        }
        meta.properties[propertyName] = prop;
    };
}
exports.createOneToDecorator = createOneToDecorator;
